/* blackhole.c */
/*
   Copyright (C) 2002
        Chris Kennedy, The Groovy Organization.

   The Blackhole is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Blackhole is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   For a copy of the GNU Library General Public License
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  or go to http://www.gnu.org
*/
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <sys/utsname.h>
#include <sys/wait.h>
#include <sysexits.h>
#include <errno.h>

#include "blackhole.h"

/* this programs name */
char *progname = "blackhole";

int main(int argc, char *argv[])
{
  int i, j;
  int maxlevel = 0;
  int checked_mailfrom = 0, checked_rcptto = 0;
  int found_mailfrom = 0, found_rcptto = 0;
  int found_ip = 0, found_sub = 0, found_charset = 0;
  int found_body = 0, found_attach = 0;
  int SHOW_CONFIG = 0, SHOW_LEVEL = 0, SHOW_RBLS = 0, STOP_WHEN_FOUND = 1;
  char *mailfrom = NULL, *rcptto = NULL, *line = NULL, *iprelay = NULL;
  char *hostrelay = NULL;
  struct utsname unamestruct;
  struct passwd *pwd;
  time_t tstamp;
  pid_t pid;
  char *name;
  FILE *debug_fp;

  /* Set umask */
  umask(077);

  /* Lower priority if set */
  if(priority > 0)
    setpriority(PRIO_PROCESS, 0, priority);

  /* stringify the version numbers */
  version = malloc(((sizeof(MAJOR_VERSION) +
                     sizeof(MINOR_VERSION) +
                     sizeof(MINOR_REVISION)) * sizeof(char)) + 4);
  snprintf(version,
           (((sizeof(MAJOR_VERSION) + sizeof(MINOR_VERSION) +
              sizeof(MINOR_REVISION)) * sizeof(char)) + 4), "%d.%d.%d",
           MAJOR_VERSION, MINOR_VERSION, MINOR_REVISION);

  /*******************************/
  /* Read the Configuration File */
  /*******************************/
  if(readconfig(SHOW_CONFIG) == 1) {
    fprintf(stderr, "%s: Error reading config!\n", progname);
    exit_mail(ERROR,iprelay,mailfrom);
  }

  /* Get Max Level */
  for(i = 0; rblhosts[i] != NULL; i++)
    maxlevel = i;

  /*******************/
  /* Check Arguments */
  /*******************/
  if(argc > 1) {
    for(i = 1; argv[i] && (i <= argc); i++) {
      if((strncmp(argv[i], "-", 1) == 0)
         && (strlen(argv[i]) == 1)) {
        usage_err(maxlevel);
        exit_mail(ERROR,iprelay,mailfrom);
      } else if(strcmp(argv[i], "-") == 0)
        continue;
      if(!strncmp(argv[i], "-debug", strlen(argv[i]))) {
        DEBUG = 1;
        if(i < (argc - 1) && strncmp(argv[i + 1], "-", 1)) {
          debug_fp = freopen(argv[i + 1], "a", stderr);
          if(debug_fp == NULL)
            fprintf(stderr,"Error opening/appending debugging logfile!\n");
          i++;
        }
        continue;
      } else if(!strncmp(argv[i], "-help", strlen(argv[i]))) {
        usage_err(maxlevel);
        exit_mail(EXIT,iprelay,mailfrom);
      } else if(!strncmp(argv[i], "-bounce", strlen(argv[i]))) {
        bouncemsg = 1;
        continue;
      } else if(!strncmp(argv[i], "-config", strlen(argv[i]))) {
        SHOW_CONFIG = 1;
        continue;
      } else if(!strncmp(argv[i], "-razor", strlen(argv[i]))) {
        userazor = 1;
        continue;
      } else if(!strncmp(argv[i], "-list", strlen(argv[i]))) {
        SHOW_RBLS = 1;
        continue;
      } else if(!strncmp(argv[i], "-log", strlen(argv[i]))) {
        use_log = 1;
        if(i < (argc - 1) && strncmp(argv[i + 1], "-", 1)) {
          if(strncmp(argv[i + 1], "stderr", strlen(argv[i + 1])) == 0)
            log_type = error;
          else if(strncmp(argv[i + 1], "mysql", strlen(argv[i + 1])) == 0)
            log_type = mysql;
          else if(strncmp(argv[i + 1], "syslog", strlen(argv[i + 1])) == 0)
            log_type = syslog;
          else if(strncmp(argv[i + 1], "stdout", strlen(argv[i + 1])) == 0)
            log_type = output;
          else {
            usage_err(maxlevel);
            fprintf(stderr, "%s: Error, bad log type given!\n", progname);
            exit_mail(ERROR,iprelay,mailfrom);
          }
          i++;
        }
        continue;
      } else if(!strncmp(argv[i], "-Sendmail", strlen(argv[i]))) {
        sendmail = 1;
        continue;
      } else if(!strncmp(argv[i], "-Resolve", strlen(argv[i]))) {
        check_sender = 1;
        continue;
      } else if(!strncmp(argv[i], "-Reverse", strlen(argv[i]))) {
        checkreverse = 1;
        continue;
      } else if(!strncmp(argv[i], "-Whitelist", strlen(argv[i]))) {
        white_list = 1;
        continue;
      } else if(!strncmp(argv[i], "-Level", strlen(argv[i]))) {
        SHOW_LEVEL = 1;
        continue;
      } else if(!strncmp(argv[i], "-total", strlen(argv[i]))) {
        STOP_WHEN_FOUND = 0;
        continue;
      } else if(!strncmp(argv[i], "-Clean", strlen(argv[i]))) {
        disinfect = 1;
        continue;
      } else if(!strncmp(argv[i], "-Virus", strlen(argv[i]))) {
        virusscan = 1;
        if(i < (argc - 1) && strncmp(argv[i + 1], "-", 1)) {
          if(strncmp(argv[i + 1], "mcafee", strlen(argv[i + 1])) == 0)
            virus_checker = McAfee;
          else if(strncmp(argv[i + 1], "sophosSDK", strlen(argv[i + 1])) == 0)
            virus_checker = SophosSDK;
          else {
            usage_err(maxlevel);
            fprintf(stderr, "%s: Error, bad scanner given!\n", progname);
            exit_mail(ERROR,iprelay,mailfrom);
          }
          i++;
        } else {
          usage_err(maxlevel);
          fprintf(stderr, "%s: Error, no scanner given!\n", progname);
          exit_mail(ERROR,iprelay,mailfrom);
        }
        continue;
      } else if(!strncmp(argv[i], "-Alert", strlen(argv[i]))) {
        virus_alert = 1;
        continue;
      } else if(!strncmp(argv[i], "-norbl", strlen(argv[i]))) {
        spam_scan = 0;
        continue;
      } else if(!strncmp(argv[i], "-Queue", strlen(argv[i]))) {
        qmail_queue = 1;
        continue;
      } else if(!strncmp(argv[i], "-BSpam", strlen(argv[i]))) {
        BODY_SCAN = 1;
        SPAM_BODY = 1;
        continue;
      } else if(!strncmp(argv[i], "-BPorn", strlen(argv[i]))) {
        BODY_SCAN = 1;
        PORN_BODY = 1;
        continue;
      } else if(!strncmp(argv[i], "-BRacist", strlen(argv[i]))) {
        BODY_SCAN = 1;
        RACIST_BODY = 1;
        continue;
      } else if(!strncmp(argv[i], "-BCustom", strlen(argv[i]))) {
        BODY_SCAN = 1;
        MY_BODY = 1;
        continue;
      } else if(!strncmp(argv[i], "-maildir", strlen(argv[i]))) {
        use_maildir = 1;
        if(i < (argc - 1) && strncmp(argv[i + 1], "-", 1)) {
          maildir = malloc(strlen(argv[i + 1]) + 1);
          if(maildir == NULL)
            fprintf(stderr, "Out of memory %s:%d\n", __FILE__, __LINE__);
          strcpy(maildir, argv[i + 1]);
          i++;
        } else {
          usage_err(maxlevel);
          fprintf(stderr, "%s: Error, no maildir given!\n", progname);
          exit_mail(ERROR,iprelay,mailfrom);
        }
        continue;
      } else if(!strncmp(argv[i], "-spool", strlen(argv[i]))) {
        SPOOLDIR = 1;
        if(i < (argc - 1) && strncmp(argv[i + 1], "-", 1)) {
          spooldir = malloc(strlen(argv[i + 1]) + 1);
          if(spooldir == NULL)
            fprintf(stderr, "Out of memory %s:%d\n", __FILE__, __LINE__);
          strcpy(spooldir, argv[i + 1]);
          i++;
        } else {
          usage_err(maxlevel);
          fprintf(stderr, "%s: Error, no spooldir given!\n", progname);
          exit_mail(ERROR,iprelay,mailfrom);
        }
        continue;
      } else if(!strncmp(argv[i], "-Delete", strlen(argv[i]))) {
        store_email = 0;
        continue;
      } else if(!strncmp(argv[i], "-Onebox", strlen(argv[i]))) {
        allinone = 1;
        continue;
      } else if(!strncmp(argv[i], "-HAscii", strlen(argv[i]))) {
        CHAR_128 = 1;
        /* Max before Spam */
        if(i < (argc - 1) && strncmp(argv[i + 1], "-", 1)) {
          for(j = 0; argv[i + 1][j] != '\0'; j++) {
            if((int) argv[i + 1][j] > 57 || (int) argv[i + 1][j] < 48) {
              /* Bad Argument Given */
              usage_err(maxlevel);
              fprintf(stderr, "%s: Error, bad option! %s\n",
                      progname, argv[i + 1]);
              exit_mail(ERROR,iprelay,mailfrom);
            }
          }
          max_ascii_score = (int) atoi(argv[i + 1]);
          i++;
        } else {
          usage_err(maxlevel);
          fprintf(stderr, "%s: Error, no char128 amount given!\n", progname);
          exit_mail(ERROR,iprelay,mailfrom);
        }
        continue;
      } else if(!strncmp(argv[i], "-Expire", strlen(argv[i]))) {
        /* Max days before Expire */
        if(i < (argc - 1) && strncmp(argv[i + 1], "-", 1)) {
          for(j = 0; argv[i + 1][j] != '\0'; j++) {
            if((int) argv[i + 1][j] > 57 || (int) argv[i + 1][j] < 48) {
              /* Bad Argument Given */
              usage_err(maxlevel);
              fprintf(stderr, "%s: Error, bad option! %s\n",
                      progname, argv[i + 1]);
              exit_mail(ERROR,iprelay,mailfrom);
            }
          }
          expire_time = (int) atoi(argv[i + 1]);
          i++;
        } else {
          usage_err(maxlevel);
          fprintf(stderr, "%s: Error, no expire time given!\n", progname);
          exit_mail(ERROR,iprelay,mailfrom);
        }
        continue;
      } else if(!strncmp(argv[i], "-MySQL", strlen(argv[i]))) {
        mysqlconfig = 1;
        /* MySQL Host */
        if(i < (argc - 1) && strncmp(argv[i + 1], "-", 1)) {
          mysql_host = malloc(strlen(argv[i + 1]) + 1);
          if(mysql_host == NULL)
            fprintf(stderr, "Out of memory %s:%d\n", __FILE__, __LINE__);
          strcpy(mysql_host, argv[i + 1]);
          i++;
        }
        continue;
      } else {
        /* Get level from command line */
        for(j = 0; argv[i][j] != '\0'; j++) {
          if((int) argv[i][j] > 57 || (int) argv[i][j] < 48) {
            /* Bad Argument Given */
            usage_err(maxlevel);
            fprintf(stderr, "%s: Error, bad option! %s\n", progname, argv[i]);
            exit_mail(ERROR,iprelay,mailfrom);
          }
        }
        level = (int) atoi(argv[i]);
      }
    }
  }

  /***************************/
  /* Show RBL Lists and Exit */
  /***************************/
  if(SHOW_RBLS == 1) {
    printf("Current RBL Lists:\n\n");
    for(i = 0; rblhosts[i] != NULL; i++)
      printf(" (%d). %s\n", i, rblhosts[i]);
    printf("\nCurrent Level: %d\n", level);
    exit_mail(EXIT,iprelay,mailfrom);
  }

  /********************************************/
  /* Show MAX and CURRENT RBL levels and Exit */
  /********************************************/
  if(SHOW_LEVEL > 0) {
    printf("Maximum: %d\n", maxlevel);
    printf("Current: %d\n", level);
    exit_mail(EXIT,iprelay,mailfrom);
  }

  /* Set level to max if more than max given */
  if(level > maxlevel)
    level = maxlevel;

  /*******************************/
  /* List the Configuration File */
  /*******************************/
  if(SHOW_CONFIG > 0) {
    if(readconfig(SHOW_CONFIG) == 1)
      fprintf(stderr, "%s: Error reading config!\n", progname);
    exit_mail(EXIT,iprelay,mailfrom);
  }

  /* get timestamp */
  timestamp = malloc(32 * sizeof(char));
  time_stamp(timestamp);

  /* Get the Hostname */
  uname(&unamestruct);
  hostname = malloc(strlen(unamestruct.nodename) + 1);
  if(hostname == NULL)
    exit_mail(ERROR,iprelay,mailfrom);
  strcpy(hostname, unamestruct.nodename);

  /* Time and Pid for Filename */
  time(&tstamp);
  pid = getpid();

  /* Get users home dir */
  pwd = getpwuid(getuid());
  homedir = malloc((strlen(pwd->pw_dir) * sizeof(char)) + 1);
  if(homedir == NULL)
    exit_mail(ERROR,iprelay,mailfrom);
  homedir = pwd->pw_dir;

  /* Get username */
  username = malloc((strlen(pwd->pw_name) * sizeof(char)) + 1);
  if(username == NULL)
    exit_mail(ERROR,iprelay,mailfrom);
  username = pwd->pw_name;

  /*************************/
  /* MySQL Database config */
  /*************************/
  if(mysqlconfig > 0) {
    if(DEBUG)
      fprintf(stderr, "Using MySQL, config file is %s\n", config_file);
    i = mysql_config();
    if(i == 0 && DEBUG)
      fprintf(stderr, "%s Updated local config with MySQL DB\n", timestamp);
  }

  /* for input line storage */
  line = malloc((255 * sizeof(char)) + 1);
  if(line == NULL) {
    fprintf(stderr,
            "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
    exit_mail(ERROR,iprelay,mailfrom);
  }

  /* ip address */
  iprelay = malloc((16 * sizeof(char)) + 1);
  if(iprelay == NULL) {
    fprintf(stderr,
            "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
    exit_mail(ERROR,iprelay,mailfrom);
  }
  iprelay[0] = '\0';

  /* host address */
  if(check_reverse > 0) {
    hostrelay = malloc((255 * sizeof(char)) + 1);
    if(hostrelay == NULL) {
      fprintf(stderr,
              "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
      exit_mail(ERROR,iprelay,mailfrom);
    }
    hostrelay[0] = '\0';
  }

  /* Message Name */
  name = (char *) malloc(strlen(hostname) + 32 + 1);
  if(name == NULL) {
    fprintf(stderr,
            "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
    exit_mail(ERROR,iprelay,mailfrom);
  }
  snprintf(name, (strlen(hostname) + 32 + 1),
           "%i.%i.%s", (int) tstamp, pid, hostname);

  /* Tmp File location + Filename */
  tmp_file = (char *) malloc(strlen(name) + strlen(spool_dir) + 10);
  if(tmp_file == NULL) {
    fprintf(stderr,
            "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
    exit_mail(ERROR,iprelay,mailfrom);
  }
  snprintf(tmp_file, (strlen(name) + strlen(spool_dir) + 10),
           "%s/msg/tmp/%s", spool_dir, name);

  /* Spam File location + Filename */
  if(use_maildir < 1 && sendmail > 0) {
    if(SPOOLDIR > 0 && strlen(spooldir) > 0) {
      spam_file = (char *) malloc(strlen(homedir) + strlen(spooldir) + 13);
      if(spam_file == NULL) {
        fprintf(stderr,
                "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
        exit_mail(ERROR,iprelay,mailfrom);
      }
      snprintf(spam_file,
               (strlen(homedir) + strlen(spooldir) + 13), "%s/%s/Inbox.Spam",
               homedir, spooldir);
    } else {
      spam_file = (char *) malloc(strlen(homedir) + 12);
      if(spam_file == NULL) {
        fprintf(stderr,
                "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
        exit_mail(ERROR,iprelay,mailfrom);
      }
      snprintf(spam_file, (strlen(homedir) + 12), "%s/Inbox.Spam", homedir);
    }
  } else {
    if(strncmp(maildir, "/", 1) == 0) {
      spam_file = (char *) malloc(strlen(name) + strlen(maildir) + 12);
      if(spam_file == NULL) {
        fprintf(stderr,
                "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
        exit_mail(ERROR,iprelay,mailfrom);
      }
      snprintf(spam_file, (strlen(name) + strlen(maildir) + 12),
               "%s/.Spam/new/%s", maildir, name);

      final_msg = (char *) malloc(strlen(name) + strlen(maildir) + 7);
      if(final_msg == NULL) {
        fprintf(stderr,
                "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
        exit_mail(ERROR,iprelay,mailfrom);
      }
      snprintf(final_msg, (strlen(name) + strlen(maildir) + 7),
               "%s/new/%s", maildir, name);
    } else {
      spam_file =
        (char *) malloc(strlen(homedir) + strlen(name) + strlen(maildir) + 13);
      if(spam_file == NULL) {
        fprintf(stderr,
                "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
        exit_mail(ERROR,iprelay,mailfrom);
      }
      snprintf(spam_file,
               (strlen(homedir) + strlen(name) + strlen(maildir) + 13),
               "%s/%s/.Spam/new/%s", homedir, maildir, name);

      final_msg =
        (char *) malloc(strlen(homedir) + strlen(name) + strlen(maildir) + 8);
      if(final_msg == NULL) {
        fprintf(stderr,
                "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
        exit_mail(ERROR,iprelay,mailfrom);
      }
      snprintf(final_msg,
               (strlen(homedir) + strlen(name) + strlen(maildir) + 8),
               "%s/%s/new/%s", homedir, maildir, name);
    }
  }

  /************************/
  /* Setup Virus Scanning */
  /************************/
  if(virusscan > 0) {
    /* Virus File location + Filename */
    if(use_maildir < 1 && sendmail > 0) {
      if(SPOOLDIR > 0 && strlen(spooldir) > 0) {
        virus_file = (char *) malloc(strlen(homedir) + strlen(spooldir) + 14);
        if(virus_file == NULL) {
          fprintf(stderr,
                  "%s: Out of memory! at %s:%d\n", progname, __FILE__,
                  __LINE__);
          exit_mail(ERROR,iprelay,mailfrom);
        }
        snprintf(virus_file,
                 (strlen(homedir) + strlen(spooldir) + 14), "%s/%s/Inbox.Virus",
                 homedir, spooldir);
      } else {
        virus_file = (char *) malloc(strlen(homedir) + 13);
        if(virus_file == NULL) {
          fprintf(stderr,
                  "%s: Out of memory! at %s:%d\n", progname, __FILE__,
                  __LINE__);
          exit_mail(ERROR,iprelay,mailfrom);
        }
        snprintf(virus_file, (strlen(homedir) + 13), "%s/Inbox.Virus", homedir);
      }
    } else {
      if(strncmp(maildir, "/", 1) == 0) {
        virus_file = (char *) malloc(strlen(name) + strlen(maildir) + 13);
        if(virus_file == NULL) {
          fprintf(stderr,
                  "%s: Out of memory! at %s:%d\n", progname, __FILE__,
                  __LINE__);
          exit_mail(ERROR,iprelay,mailfrom);
        }
        snprintf(virus_file, (strlen(name) + strlen(maildir) + 13),
                 "%s/.Virus/new/%s", maildir, name);
      } else {
        virus_file =
          (char *) malloc(strlen(homedir) + strlen(name) + strlen(maildir) +
                          14);
        if(virus_file == NULL) {
          fprintf(stderr,
                  "%s: Out of memory! at %s:%d\n", progname, __FILE__,
                  __LINE__);
          exit_mail(ERROR,iprelay,mailfrom);
        }
        snprintf(virus_file,
                 (strlen(homedir) + strlen(name) + strlen(maildir) + 14),
                 "%s/%s/.Virus/new/%s", homedir, maildir, name);
      }
    }
  }

  /**********************************/
  /* Open TMP File to store message */
  /**********************************/
  if(DEBUG)
    fprintf(stderr, "Opening: %s\n", tmp_file);

  tmp_msg = fopen(tmp_file, "w");
  if(tmp_msg == NULL) {
    fprintf(stderr, "%s: Couldn't open %s file!\n", progname, tmp_file);
    exit_mail(ERROR,iprelay,mailfrom);
  }

  /* Qmail SENDER and RECIPIENT */
  if(sendmail < 1) {
    /* Get the Sender */
    mailfrom = malloc((sizeof(getenv("SENDER")) * sizeof(char)) + 1);
    if(mailfrom == NULL) {
      fprintf(stderr,
              "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
      fclose(tmp_msg);
      unlink(tmp_file);
      exit_mail(ERROR,iprelay,mailfrom);
    }
    mailfrom = (char *) getenv("SENDER");
    found_mailfrom = 1;

    /* Get the Recipient */
    rcptto = malloc((sizeof(getenv("RECIPIENT")) * sizeof(char)) + 1);
    if(rcptto == NULL) {
      fprintf(stderr,
              "%s: Out of memory! at %s:%d\n", progname, __FILE__, __LINE__);
      fclose(tmp_msg);
      unlink(tmp_file);
      exit_mail(ERROR,iprelay,mailfrom);
    }
    rcptto = (char *) getenv("RECIPIENT");
    found_rcptto = 1;
  }

  /**********************************/
  /* Read entire message from stdin */
  /**********************************/
  while(fgets(line, 255, stdin)) {
    /* Qmail Queue output */
    if(qmail_queue)
      fputs(line,stdout);

    /* Write to tmp file */
    fputs(line,tmp_msg);

    if((strncmp(line, "\n", 1) == 0) && iprelay[0] != '\0') {
      found_body = 1;

      /* White List in effect, if haven't been approved yet your gone */
      if(white_list > 0)
        match = MATCH_WHITE_LIST;
    }

    /* Ascii char check, see if they are outside latin Charsets */
    if(CHAR_128 > 0 && found_attach == 0 && match == NO_MATCH)
      if(ascii_128(line) == 1)
        match = MATCH_ASCII_128;

    /* Body Checking, skips attachments */
    if(BODY_SCAN > 0 && match == NO_MATCH && found_body == 1) {
      if(found_attach == 0
         && strncmp(line, "Content-Disposition: attachment", 31) != 0) {
        if((j = check_body(line)) > 0)
          match = j;
      } else if(found_attach == 0)
        found_attach = 1;
    }

    if(found_body == 0) {
      /* Found "From " */
      if(sendmail > 0) {
        if(found_mailfrom == 0) {
          if(!strncmp(line, "From ", 5)) {
            mailfrom = (char *) malloc(strlen(line) + 1);
            if(mailfrom == NULL) {
              fprintf(stderr,
                    "%s: Out of memory! at %s:%d\n", progname, __FILE__,
                    __LINE__);
              fclose(tmp_msg);
              unlink(tmp_file);
              exit_mail(ERROR,iprelay,mailfrom);
            }
            for(j = 5, i = 0; line[j] != ' '; j++, i++)
              mailfrom[i] = line[j];
            mailfrom[i] = '\0';
            found_mailfrom = 1;
            if(DEBUG)
              printf("MAILFROM: %s\n", mailfrom);
          }
          /* Found "        for <" */
        } else if(found_rcptto == 0) {
          if(!strncmp(line, "\tfor <", 6)) {
            rcptto = (char *) malloc(strlen(line) + 1);
            if(rcptto == NULL) {
              fprintf(stderr,
                    "%s: Out of memory! at %s:%d\n", progname, __FILE__,
                    __LINE__);
              fclose(tmp_msg);
              unlink(tmp_file);
              exit_mail(ERROR,iprelay,mailfrom);
            }
            for(j = 6, i = 0; line[j] != '>'; j++, i++)
              rcptto[i] = line[j];
            rcptto[i] = '\0';
            found_rcptto = 1;
            if(DEBUG)
              printf("RCPTTO: %s\n", rcptto);
          }
        }
      }
      /* Found "Received: from " */
      if(found_ip == 0 && !strncmp(line, "Received: from ", 15)) {
        if(DEBUG)
          fprintf(stderr, "Checking for Relay IP:\n");

        if(!regexip(line, iprelay, hostrelay)) {
          if(DEBUG)
            fprintf(stderr, "Checking My Relays:\n");

          /* Check against local Relays */
          if(checkmyrelay(iprelay, myrelays)) {
            found_ip = 1;
            /* Check Relay */
            if(match == NO_MATCH && iprelay != NULL) {
              if(DEBUG)
                fprintf(stderr, "Checking Good Relay IPs:\n");

              /* Good Relay Checks */
              if(checkgoodrelay(iprelay, goodrelays))
                exit_mail(PERMIT, iprelay, mailfrom);

              if(checkreverse > 0) {
                if(DEBUG)
                  fprintf(stderr, "Checking Reverse DNS:\n");
                if(check_reverse(iprelay, hostrelay) == 0)
                  match = MATCH_REVERSE;
              }

              if(DEBUG)
                fprintf(stderr, "Checking Bad Relay IPs:\n");

              /* Bad Relay Checks */
              if(match == NO_MATCH && checkbadrelay(iprelay, badrelays))
                match = MATCH_RELAY;

              /* RBL Checks */
              if(spam_scan > 0 && match == NO_MATCH) {
                if(DEBUG)
                  fprintf(stderr, "Checking RBL Lists:\n");

                if(rbllookup(iprelay, rblhosts, level, STOP_WHEN_FOUND))
                  match = MATCH_BLACKHOLE;
              }
            }
          } else
            iprelay[0] = '\0';
        }
      } else if(found_sub == 0 && found_ip == 1 &&
                (strncmp(line, "Subject:", 8) == 0 ||
                 strncmp(line, "SUBJECT:", 8) == 0 ||
                 strncmp(line, "subject:", 8) == 0)) {
        /* Found "Subject:" */
        if(DEBUG)
          fprintf(stderr, "Checking Subject line:\n %s", line);

        /* Matched Subject */
        if(match == NO_MATCH && checkbadsub(line, badsubject)) {
          match = MATCH_SUBJECT;
        }

        /* Found Subject */
        found_sub = 1;
      } else if(found_charset == 0 && found_ip == 1 && charsets[0] != NULL &&
                (strstr(line, "charset=") != NULL ||
                 strstr(line, "CHARSET=") != NULL ||
                 strstr(line, "Charset=") != NULL)) {
        if(DEBUG)
          fprintf(stderr, "Checking charset:\n");
        /* Found "charset=" */
        if(match == NO_MATCH && check_charset(line, charsets) == 0) {
          if(DEBUG)
            fprintf(stderr, " %s", line);

          match = MATCH_CHARSET;
        }

        /* Found Subject */
        found_charset = 1;
      }

      /* Check SENDER if mailfrom found */
      if(checked_mailfrom == 0 && found_mailfrom == 1 && mailfrom != NULL) {
        for(i = 0; mailfrom[i] != '\0'; i++)
          mailfrom[i] = tolower(mailfrom[i]);
        mailfrom[i] = '\0';

        if(DEBUG) {
          fprintf(stderr, "Mail From: %s\n", mailfrom);
          fprintf(stderr, "Checking Good Email Addresses:\n");
        }

        /* Good Email Check */
        if(checkgoodemail(mailfrom, goodemail))
          exit_mail(PERMIT, iprelay, mailfrom);

        if(DEBUG)
          fprintf(stderr, "Checking Bad Email Addresses:\n");

        /* Bad Email Check */
        if(match == NO_MATCH && checkbademail(mailfrom, bademail))
          match = MATCH_EMAIL;

        if(match == NO_MATCH && check_sender > 0)
          if(check_dns(mailfrom) == 0)
            match = MATCH_SENDER_DNS;

        /* Avoid repeating loop */
        checked_mailfrom = 1;
      }

      /* Check RECIPIENT if rcptto found */
      if(checked_rcptto == 0 && found_rcptto == 1 && rcptto != NULL) {
        for(i = 0; rcptto[i] != '\0'; i++)
          rcptto[i] = tolower(rcptto[i]);
        rcptto[i] = '\0';

        if(DEBUG)
          fprintf(stderr, "Checking My Email Addresses:\n");

        /* My Email Check, allowed recipients */
        if(match == NO_MATCH)
          if(myemail[0] != NULL && checkmyemail(rcptto, myemail) == 0)
            match = MATCH_MY_EMAIL;
        checked_rcptto = 1;
      }
    }
  }

  /*********************************************/
  /* Deny just means to check still with Razor */
  /*********************************************/
  exit_mail(DENY, iprelay, mailfrom);

  /* Should never get Here */
  return 0;
}

void usage_err(int maxlevel)
{
  fprintf(stderr, "Blackhole Version-%s by ", version);
  fprintf(stderr, "Chris Kennedy (C) 2001, 2002\n\n");
  fprintf(stderr,
          "Usage: %s [level] [-d [log]][-h][-li][-L][-c][-b][-t][-n][-S][-C]\n",
          progname);
  fprintf(stderr, "\t\t[-V type][-A][-B{SRPC}][-m maildir][-D][-Q][-E][-W]\n");
  fprintf(stderr, "\t\t[-lo [type] [-M [host]] [-s spooldir][-O][-HA max]\n");
  fprintf(stderr, "\t\t[-Res][-Rev][-r]\n\n");
  fprintf(stderr,
          "\tlevel     Level of RBL checking to do (0-%d).\n", maxlevel);
  fprintf(stderr, "\n");
  fprintf(stderr, "\t-help     This output you see here.\n");
  fprintf(stderr, "\t-debug [logfile] Very verbose output for testing.\n");
  fprintf(stderr, "\t-config   Show current config from .blackhole file.\n");
  fprintf(stderr, "\t-Level    Show maximum possible level and current one.\n");
  fprintf(stderr, "\t-list     List current RBL Lists in order of levels.\n");
  fprintf(stderr,
          "\t-log [type] Print out logging info when actions happen.\n");
  fprintf(stderr, "\t-bounce   bounce messages blackholed back to sender.\n");
  fprintf(stderr,
          "\t-Delete   Delete all Spam and Virus email, don't store.\n");
  fprintf(stderr, "\t-norbl    No RBL Lists, skip using them for checking.\n");
  fprintf(stderr,
          "\t-total    Use level for maxnumber of RBL Lists matched.\n");
  fprintf(stderr, "\t-BSpam    Check body for known Spam patterns.\n");
  fprintf(stderr, "\t-BRacist  Check body for known Racist patterns.\n");
  fprintf(stderr, "\t-BPorn    Check body for known Porn/Sex patterns.\n");
  fprintf(stderr, "\t-BCustom  Check body for [my_body] custom patterns.\n");
  fprintf(stderr, "\t-Virus {scanner} Check for viruses, use .Virus mbox.\n");
  fprintf(stderr, "\t-Alert    Send an alert back to the virus sender.\n");
  fprintf(stderr, "\t-Clean    Remove viruses from email, disinfect.\n");
  fprintf(stderr, "\t-maildir {dir} Change Maildir (default is mail).\n");
  fprintf(stderr,
          "\t-Queue    Use with qmail-qfilter to run in Qmail queue.\n");
  fprintf(stderr, "\t-Sendmail Use with Sendmail, writes to Inbox.Spam.\n");
  fprintf(stderr, "\t-spool {dir} mbox ('-s Mail' gets ~/Mail/Inbox.Spam).\n");
  fprintf(stderr, "\t-Resolve  Check that senders return address is valid.\n");
  fprintf(stderr,
          "\t-Reverse  Check reverse vs forward DNS match for relay.\n");
  fprintf(stderr,
          "\t-Whitelist Only allow good email/relay, deny all other.\n");
  fprintf(stderr, "\t-MySQL [host] Config in Database, localhost default.\n");
  fprintf(stderr, "\t-Onebox   Change Subject: sends to the main mail box\n");
  fprintf(stderr, "\t\t\tinstead of .Spam or .Virus, works on all servers.\n");
  fprintf(stderr, "\t-HAscii {max} ASCII characters above 128, max allowed.\n");
  fprintf(stderr, "\t-Expire [days] Days to delete .Spam and .Virus mail.\n");
  fprintf(stderr, "\t-razor    Use Razor to check, must get Razor first.\n");
  fprintf(stderr, "\nVirus scanners:\n\tSophosSDK\n\tmcafee\n");
  fprintf(stderr, "\nLogging types:\n\tsyslog\tmysql\n\tstderr\tstdout\n\n");
}

int time_stamp(char timestamp[])
{
  time_t current;
  struct tm *t = localtime(&current);
  int year;

  extern char *tzname[2];

  char *weekdays[] = { "Sun", "Mon", "Tue", "Wed", "Thur", "Fri", "Sat" };
  char *months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug",
    "Sep", "Oct", "Nov", "Dec"
  };

  current = time(NULL);
  localtime(&current);

  year = t->tm_year + 1900;

  snprintf(timestamp, (32 * sizeof(char)),
           "%s %s %2d %02d:%02d:%02d %s %d",
           weekdays[t->tm_wday], months[t->tm_mon],
           t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec,
           tzname[t->tm_isdst], year);

  return 0;
}

int readconfig(int SHOW_CONFIG)
{
  FILE *config;
  int i = 0, j = 0;
  int line_cnt = 0, opt_cnt = 0;
  enum sections where = 0;
  char *section = NULL, *buffer = NULL;
  char *line;

  /* Open Config File, tries mbox/.blackhole,.blackhole,/etc/blackhole.conf */
  config = fopen(config_file, "r");

  if(config == NULL) {
    config = fopen(alt_config_file, "r");

    if(config == NULL) {
      config = fopen(default_cfg, "r");

      /* Create a blank config if none exists */
      if(config == NULL) {
        config = fopen(config_file, "w+");
        if(config == NULL) {
          config = fopen(alt_config_file, "w+");
          if(config == NULL)
            return (1);
        }
        fprintf(config, "# BlackHole Default Config\n");
        fprintf(config, "[level]\n");
        fprintf(config, "7\n");
      }
      config_file = malloc(strlen(default_cfg) + 1);
      strcpy(config_file, default_cfg);
    } else {
      config_file = malloc(strlen(alt_config_file) + 1);
      strcpy(config_file, alt_config_file);
    }
  }

  section = (char *) malloc((MAX_CONFIG_LINE * sizeof(char)) + 1);
  if(section == NULL)
    return (1);

  line = (char *) malloc((MAX_CONFIG_LINE * sizeof(char)) + 1);
  if(line == NULL)
    return (1);

  buffer = (char *) malloc((MAX_CONFIG_LINE * sizeof(char)) + 1);
  if(buffer == NULL)
    return (1);

  i = 0;
  while((fgets(line, MAX_CONFIG_LINE, config)) != NULL) {
    /* Get Section */
    if(strncmp(line, "[", 1) == 0) {
      line_cnt++;
      count = 0;
      opt_cnt = 0;
      for(i = 0, j = 1; line[j] != ']' && line[j] != '\0'; i++, j++)
        section[i] = line[j];
      if(line[j] != ']') {
        continue;
        section[0] = '\0';
      }
      section[i] = '\0';

      /* Which Section? */
      if((where = get_section(section)) == 0) {
        fprintf(stderr, "\n%s: Illegal Section %s!\n", config_file, section);
      }

      if(SHOW_CONFIG && section[0] != '\0') {
        printf("[%s]\n", section);
        if((line_cnt % 25) == 0) {
          fprintf(stderr, " -- More #%d --", line_cnt);
          while(fgetc(stdin) != '\n');
        }
      }
      continue;
      /* Skip Comment */
    } else if(strncmp(line, "#", 1) == 0) {
      continue;
      /* Skip Empty Line */
    } else if(strncmp(line, "\n", 1) == 0) {
      continue;
      /* Not a section */
    } else if(where == 0) {
      continue;
      /* copy into buffer */
    } else {
      line_cnt++;
      opt_cnt++;
      if(opt_cnt > (maxoptions - 1))
        continue;
      for(i = 0; line[i] != '\n'; i++)
        buffer[i] = line[i];
      buffer[i] = '\0';
      count++;
    }

    /* Print Config */
    if(SHOW_CONFIG && buffer[0] != '\0') {
      printf("%s\n", buffer);
      if((line_cnt % 25) == 0) {
        fprintf(stderr, " -- More #%d --", line_cnt);
        while(fgetc(stdin) != '\n');
      }
      continue;
    }

    /* fill associative arrays */
    if(buffer[0] != '\0') {
      if(add_option(where, buffer) == 1)
        return 1;
    }
  }
  fclose(config);
  if(SHOW_CONFIG)
    printf("\n");

  if(section != NULL)
    free(section);
  if(buffer != NULL)
    free(buffer);

  return 0;
}

int get_section(char *section)
{
  enum sections where = 0;

  /* get which section */
  if(!strcmp(section, "my_relay"))
    where = MY_RELAY;
  else if(!strcmp(section, "good_relay"))
    where = GOOD_RELAY;
  else if(!strcmp(section, "bad_relay"))
    where = BAD_RELAY;
  else if(!strcmp(section, "good_email"))
    where = GOOD_EMAIL;
  else if(!strcmp(section, "bad_email"))
    where = BAD_EMAIL;
  else if(!strcmp(section, "bad_subject"))
    where = BAD_SUB;
  else if(!strcmp(section, "rbl_hosts"))
    where = RBL_HOSTS;
  else if(!strcmp(section, "level"))
    where = CONFLEVEL;
  else if(!strcmp(section, "bounce"))
    where = BOUNCE_550;
  else if(!strcmp(section, "bounce_msg"))
    where = BOUNCE_MES;
  else if(!strcmp(section, "rbl_check"))
    where = SPAM_CHK;
  else if(!strcmp(section, "vscan"))
    where = VIRUS_CHK;
  else if(!strcmp(section, "valert"))
    where = VIRUS_MSG;
  else if(!strcmp(section, "vclean"))
    where = VIRUS_CL;
  else if(!strcmp(section, "body_check_spam"))
    where = BODY_CHK_S;
  else if(!strcmp(section, "body_check_porn"))
    where = BODY_CHK_P;
  else if(!strcmp(section, "body_check_racist"))
    where = BODY_CHK_R;
  else if(!strcmp(section, "white_list"))
    where = WHITE_LIST_CHK;
  else if(!strcmp(section, "my_email"))
    where = MY_EMAIL;
  else if(!strcmp(section, "check_dns"))
    where = DNS_CHK;
  else if(!strcmp(section, "one_box"))
    where = ALLONE;
  else if(!strcmp(section, "body_check"))
    where = BODY_CHK;
  else if(!strcmp(section, "my_body"))
    where = MY_BODY_CHK;
  else if(!strcmp(section, "charsets"))
    where = CHARSETS;
  else if(!strcmp(section, "ascii_128"))
    where = MAX_ASCII_128;
  else if(!strcmp(section, "expire"))
    where = EXPIRE;
  else if(!strcmp(section, "check_reverse"))
    where = REVERSE;
  else if(!strcmp(section, "razor"))
    where = RAZOR;
  else if(!strcmp(section, "bad_subject_action"))
    where = A_MATCH_SUBJECT;
  else if(!strcmp(section, "bad_email_action"))
    where = A_MATCH_EMAIL;
  else if(!strcmp(section, "bad_relay_action"))
    where = A_MATCH_RELAY;
  else if(!strcmp(section, "rbl_check_action"))
    where = A_MATCH_BLACKHOLE;
  else if(!strcmp(section, "body_check_spam_action"))
    where = A_MATCH_BODY_SPAM;
  else if(!strcmp(section, "body_check_porn_action"))
    where = A_MATCH_BODY_PORN;
  else if(!strcmp(section, "body_check_racist_action"))
    where = A_MATCH_BODY_RACIST;
  else if(!strcmp(section, "white_list_action"))
    where = A_MATCH_WHITE_LIST;
  else if(!strcmp(section, "my_email_action"))
    where = A_MATCH_MY_EMAIL;
  else if(!strcmp(section, "check_dns_action"))
    where = A_MATCH_SENDER_DNS;
  else if(!strcmp(section, "body_check_action"))
    where = A_MATCH_MY_BODY;
  else if(!strcmp(section, "charsets_action"))
    where = A_MATCH_CHARSET;
  else if(!strcmp(section, "ascii_128_action"))
    where = A_MATCH_ASCII_128;
  else if(!strcmp(section, "check_reverse_action"))
    where = A_MATCH_REVERSE;
  else if(!strcmp(section, "razor_action"))
    where = A_MATCH_RAZOR;
  else {
    where = 0;
  }

  /* Return Section */
  return where;
}

int add_option(int where_cur, char *buffer)
{
  enum sections where = where_cur;
  int j;

  switch (where) {
  case MY_RELAY:
    myrelays[count - 1] = (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(myrelays[count - 1], buffer);
    break;
  case MY_EMAIL:
    myemail[count - 1] = (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(myemail[count - 1], buffer);
    break;
  case GOOD_EMAIL:
    goodemail[count - 1] = (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(goodemail[count - 1], buffer);
    break;
  case BAD_EMAIL:
    bademail[count - 1] = (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(bademail[count - 1], buffer);
    break;
  case GOOD_RELAY:
    goodrelays[count - 1] =
      (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(goodrelays[count - 1], buffer);
    break;
  case BAD_RELAY:
    badrelays[count - 1] = (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(badrelays[count - 1], buffer);
    break;
  case BAD_SUB:
    badsubject[count - 1] =
      (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(badsubject[count - 1], buffer);
    break;
  case RBL_HOSTS:
    rblhosts[count - 1] = (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(rblhosts[count - 1], buffer);
    rblhosts[count] = (char *) malloc(sizeof(char));
    rblhosts[count] = NULL;
    break;
  case MY_BODY_CHK:
    mybody[count - 1] = (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(mybody[count - 1], buffer);
    break;
  case CHARSETS:
    charsets[count - 1] = (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(charsets[count - 1], buffer);
    break;
  case BOUNCE_MES:
    bounce_msg = (char *) malloc((strlen(buffer) * sizeof(char)) + 1);
    strcpy(bounce_msg, buffer);
    break;
  case SPAM_CHK:
    if(buffer[0] == '0' || buffer[0] == '-' || buffer[0] == '1')
      spam_scan = (int) atoi(buffer);
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      spam_scan = 1;
    else 
      spam_scan = 0;

    break;
  case BODY_CHK:
    if(buffer[0] == '-')
      MY_BODY = 0;
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      MY_BODY = 1;
    else {
      for(j = 0; buffer[j] != '\0' && buffer[j] != '\n' && 
	buffer[j] != ' '; j++) {
        if((int) buffer[j] > 57 || (int) buffer[j] < 48) {
          /* Not a number */
          fprintf(stderr, "Error, bad option for custom body! %s\n", buffer);
          level = 0;
          break;
        }
      }
      MY_BODY = (int) atoi(buffer);
    }
    if(MY_BODY > 0)
      BODY_SCAN = 1;
    break;
  case BODY_CHK_S:
    if(buffer[0] == '-')
      SPAM_BODY = 0;
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      SPAM_BODY = 1;
    else {
      for(j = 0; buffer[j] != '\0' && buffer[j] != '\n' && 
	buffer[j] != ' '; j++) {
        if((int) buffer[j] > 57 || (int) buffer[j] < 48) {
          /* Not a number */
          fprintf(stderr, "Error, bad option for spam body! %s\n", buffer);
          level = 0;
          break;
        }
      }
      SPAM_BODY = (int) atoi(buffer);
    }
    if(SPAM_BODY > 0)
      BODY_SCAN = 1;
    break;
  case BODY_CHK_P:
    if(buffer[0] == '-')
      PORN_BODY = 0;
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      PORN_BODY = 1;
    else {
      for(j = 0; buffer[j] != '\0' && buffer[j] != '\n' && 
	buffer[j] != ' '; j++) {
        if((int) buffer[j] > 57 || (int) buffer[j] < 48) {
          /* Not a number */
          fprintf(stderr, "Error, bad option for porn body! %s\n", buffer);
          level = 0;
          break;
        }
      }
      PORN_BODY = (int) atoi(buffer);
    }
    if(PORN_BODY > 0)
      BODY_SCAN = 1;
    break;
  case BODY_CHK_R:
    if(buffer[0] == '-')
      RACIST_BODY = 0;
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      RACIST_BODY = 1;
    else {
      for(j = 0; buffer[j] != '\0' && buffer[j] != '\n' && 
	buffer[j] != ' '; j++) {
        if((int) buffer[j] > 57 || (int) buffer[j] < 48) {
          /* Not a number */
          fprintf(stderr, "Error, bad option for racist body! %s\n", buffer);
          level = 0;
          break;
        }
      }
      RACIST_BODY = (int) atoi(buffer);
    }
    if(RACIST_BODY > 0)
      BODY_SCAN = 1;
    break;
  case VIRUS_CHK:
    if(buffer[0] == '0' || buffer[0] == '-' || buffer[0] == '1')
      virusscan = (int) atoi(buffer);
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      virusscan = 1;
    else
      virusscan = 0;
    break;
  case VIRUS_MSG:
    if(buffer[0] == '0' || buffer[0] == '-' || buffer[0] == '1')
      virus_alert = (int) atoi(buffer);
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      virus_alert = 1;
    else 
      virus_alert = 0;
    break;
  case VIRUS_CL:
    if(buffer[0] == '0' || buffer[0] == '-' || buffer[0] == '1')
      disinfect = (int) atoi(buffer);
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      disinfect = 1;
    else
      disinfect = 0;
    break;
  case BOUNCE_550:
    if(buffer[0] == '0' || buffer[0] == '-' || buffer[0] == '1')
      bouncemsg = (int) atoi(buffer);
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      bouncemsg = 1;
    else
      bouncemsg = 0;
    break;
  case RAZOR:
    if(buffer[0] == '0' || buffer[0] == '-' || buffer[0] == '1')
      userazor = (int) atoi(buffer);
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      userazor = 1;
    else
      userazor = 0;
    break;
  case CONFLEVEL:
    if(buffer[0] == '-')
      level = 0;
    else {
      for(j = 0; buffer[j] != '\0' && buffer[j] != '\n' && 
	buffer[j] != ' '; j++) {
        if((int) buffer[j] > 57 || (int) buffer[j] < 48) {
          /* Not a number */
          fprintf(stderr, "Error, bad option for level! %s\n", buffer);
          level = 0;
          break;
        }
      }
      level = (int) atoi(buffer);
    }
    break;
  case MAX_ASCII_128:
    if(buffer[0] == '-')
      max_ascii_score = 0;
    else {
      for(j = 0; buffer[j] != '\0' && buffer[j] != '\n' && 
	buffer[j] != ' '; j++) {
        if((int) buffer[j] > 57 || (int) buffer[j] < 48) {
          /* Not a number */
          fprintf(stderr, "Error, bad option for max ascii 128! %s\n", buffer);
          level = 0;
          break;
        }
      }
      max_ascii_score = (int) atoi(buffer);
    }
    if(max_ascii_score > 0)
      CHAR_128 = 1;
    break;
  case WHITE_LIST_CHK:
    if(buffer[0] == '0' || buffer[0] == '-' || buffer[0] == '1')
      white_list = (int) atoi(buffer);
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      white_list = 1;
    else
      white_list = 0;
    break;
  case DNS_CHK:
    if(buffer[0] == '0' || buffer[0] == '-' || buffer[0] == '1')
      check_sender = (int) atoi(buffer);
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      check_sender = 1;
    else
      check_sender = 0;
    break;
  case REVERSE:
    if(buffer[0] == '0' || buffer[0] == '-' || buffer[0] == '1')
      checkreverse = (int) atoi(buffer);
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      checkreverse = 1;
    else
      checkreverse = 0;
    break;
  case ALLONE:
    if(buffer[0] == '0' || buffer[0] == '-' || buffer[0] == '1')
      allinone = (int) atoi(buffer);
    else if((strncmp(buffer,"on",2) == 0) || (strncmp(buffer,"true",4) == 0))
      allinone = 1;
    else
      allinone = 0;
    break;
  case EXPIRE:
    if(buffer[0] == '-')
      expire_time = 0;
    else {
      for(j = 0; buffer[j] != '\0' && buffer[j] != '\n' && 
	buffer[j] != ' '; j++) {
        if((int) buffer[j] > 57 || (int) buffer[j] < 48) {
          /* Not a number */
          fprintf(stderr, "Error, bad option for max expire! %s\n", buffer);
          level = 0;
          break;
        }
      }
      expire_time = (int) atoi(buffer);
    }
    break;
  case A_MATCH_SUBJECT:
    get_action(where,buffer);
    break;
  case A_MATCH_EMAIL:
    get_action(where,buffer);
    break;
  case A_MATCH_RELAY:
    get_action(where,buffer);
    break;
  case A_MATCH_BLACKHOLE:
    get_action(where,buffer);
    break;
  case A_MATCH_BODY_SPAM:
    get_action(where,buffer);
    break;
  case A_MATCH_BODY_PORN:
    get_action(where,buffer);
    break;
  case A_MATCH_BODY_RACIST:
    get_action(where,buffer);
    break;
  case A_MATCH_WHITE_LIST:
    get_action(where,buffer);
    break;
  case A_MATCH_MY_EMAIL:
    get_action(where,buffer);
    break;
  case A_MATCH_SENDER_DNS:
    get_action(where,buffer);
    break;
  case A_MATCH_MY_BODY:
    get_action(where,buffer);
    break;
  case A_MATCH_CHARSET:
    get_action(where,buffer);
    break;
  case A_MATCH_ASCII_128:
    get_action(where,buffer);
    break;
  case A_MATCH_REVERSE:
    get_action(where,buffer);
    break;
  case A_MATCH_RAZOR:
    get_action(where,buffer);
    break;
  default:
    fprintf(stderr, "%s: Error, No such section #%d!\n", config_file, where);
    return 1;
  }
  return 0;
}

int get_action(int check_id, char *buffer) {
  char *name, *value;
  int i = 0, j = 0;

  name = malloc(strlen(buffer) + 1);
  if(name == NULL)
    return 1;
  value = malloc(strlen(buffer) + 1);
  if(value == NULL)
    return 1;

  /* Get name and value pair */
  while(buffer[i] != '\0' && buffer[i] != ' ' && buffer[i] != '=') {
    name[j++] = buffer[i];
    i++;
  }
  name[j+1] = '\0';
  while(buffer[i] != '\0' && (buffer[i] == ' ' || buffer[i] == '='))
    i++;
  j=0;
  while(buffer[i] != '\0') {
    value[j++] = buffer[i];
    i++;
  }
  value[j+1] = '\0';

  /* Check what name is being setup */
  if(strcmp(name,"active") == 0) {
    if(value[0] == '0' || value[0] == '-' || value[0] == '1')
      bh_action[check_id].active = (int) atoi(value);
    else if((strncmp(value,"on",2) == 0) || (strncmp(value,"true",4) == 0))
      bh_action[check_id].active = 1;
    else
      bh_action[check_id].active = 0;
  } else if(strcmp(name,"one_box") == 0) {
    if(value[0] == '0' || value[0] == '-' || value[0] == '1')
      bh_action[check_id].one_box = (int) atoi(value);
    else if((strncmp(value,"on",2) == 0) || (strncmp(value,"true",4) == 0))
      bh_action[check_id].one_box = 1;
    else
      bh_action[check_id].one_box = 0;
  } else if(strcmp(name,"delete") == 0) {
    if(value[0] == '0' || value[0] == '-' || value[0] == '1')
      bh_action[check_id].delete = (int) atoi(value);
    else if((strncmp(value,"on",2) == 0) || (strncmp(value,"true",4) == 0))
      bh_action[check_id].delete = 1;
    else
      bh_action[check_id].delete = 0;
  } else if(strcmp(name,"bounce_msg") == 0) {
    bh_action[check_id].bounce_msg = 
	malloc((strlen(value) * sizeof(char)) + 1);
    strcpy(bh_action[check_id].bounce_msg, value);
  } else if(strcmp(name,"bounce") == 0) {
    if(value[0] == '0' || value[0] == '-' || value[0] == '1')
      bh_action[check_id].bounce = (int) atoi(value);
    else if((strncmp(value,"on",2) == 0) || (strncmp(value,"true",4) == 0))
      bh_action[check_id].bounce = 1;
    else
      bh_action[check_id].bounce = 0;
  } else if(strcmp(name,"score") == 0) {
    bh_action[check_id].score = (float) atof(value);
  } else if(strcmp(name,"passthru") == 0) {
    if(value[0] == '0' || value[0] == '-' || value[0] == '1')
      bh_action[check_id].passthru = (int) atoi(value);
    else if((strncmp(value,"on",2) == 0) || (strncmp(value,"true",4) == 0))
      bh_action[check_id].passthru = 1;
    else
      bh_action[check_id].passthru = 0;
  } 

  return 0; 
}

void exit_mail(int action, char iprelay[16], char *mailfrom)
{
  extern int errno;
  int virus_ret = 0;
  char *mbox = NULL;

  if(action == PERMIT)
    match = 0;
  else if(action == EXIT)
    exit(0);
  else if(action == ERROR) {
    if(sendmail > 0)
      exit(EX_TEMPFAIL);
    else if(qmail_queue > 0)
      exit(51);
    else
      exit(0);
  }

  /* Read rest of stdin to file or output (qmail-queue) */
  if(tmp_msg != NULL) {
    char buffer[256];
    while(fgets(buffer, 255, stdin) != '\0') {
      if(qmail_queue > 0) {
        fputs(buffer,stdout);
      }
      fputs(buffer,tmp_msg);
    }
    fclose(tmp_msg);
  }

  /* Razor Check the Email */
  if(action == DENY && userazor > 0 && match == 0) {
    if(razor_check() == 1) {
       match = 15; 
    }
  }

  /* Action Check, see if an individual action is defined for this match */
  if(bh_action[match].active > 0) {
    if(bh_action[match].one_box == 1) 
      allinone = 1;
    if(bh_action[match].delete == 1)
      store_email = 0; 
    if(bh_action[match].bounce == 1)
      bouncemsg = 1;
    if(bh_action[match].bounce_msg != NULL) {
      bounce_msg = malloc(strlen(bh_action[match].bounce_msg) + 1);
      if(bounce_msg != NULL) 
        strcpy(bounce_msg,bh_action[match].bounce_msg);
    }
  }

  /* Global Spool mbox for Sendmail/Postfix */
  if(use_maildir < 1 && sendmail > 0) {
    if(sendmail_dir[0] == '/') {
      mbox = malloc(strlen(username) + strlen(sendmail_dir) + 2);
      if(mbox == NULL)
        exit(EX_TEMPFAIL);
      snprintf(mbox, (strlen(username) + strlen(sendmail_dir) + 2),
        "%s/%s", sendmail_dir, username);
    } else {
      int i;
      for(i=0;sendmail_dir[i] != '\0';i++);
      if(sendmail_dir[i] == '/') {
        mbox = malloc(strlen(homedir) + strlen(sendmail_dir) + 7 + 1);
        if(mbox == NULL)
          exit(EX_TEMPFAIL);
        snprintf(mbox, (strlen(homedir) + strlen(sendmail_dir) + 7 + 1),
          "%s/%s/Inbox", homedir, sendmail_dir);
      } else {
        mbox = malloc(strlen(homedir) + strlen(sendmail_dir) + 1 + 1);
        if(mbox == NULL)
          exit(EX_TEMPFAIL);
        snprintf(mbox, (strlen(homedir) + strlen(sendmail_dir) + 1 + 1),
          "%s/%s", homedir, sendmail_dir);
      }
    }
  /* Expire Maildir format mailboxes */
  } else if(expire_time > 0) {
    expire(".Spam/cur", expire_time);
    expire(".Spam/new", expire_time);
    if(virusscan > 0) {
      expire(".Virus/cur", expire_time);
      expire(".Virus/new", expire_time);
    }
  }

  /* Virus Scan */
  if(virusscan > 0) {
    /* Run Virus Scan */
    virus_ret = virus_scan();
    if(found_virus == 0)
      virus_type = NULL;

    if(virus_ret > 0) {
      if(virus_alert > 0) {
        fprintf(stdout, "You are sending a Virus: %s\n", virus_type);
        fprintf(stdout, "This has been blocked by our system,\n");
        fprintf(stdout, "please use a virus scanner to fix your system. ");
        fprintf(stdout, "(#5.1.1)\n");
      }

      if(store_email > 0) {
        if(use_maildir < 1 && sendmail > 0) {
          if(allinone > 0) {
            if(send_mail_box(mbox, tmp_file, mailfrom, iprelay, virus_ret)
               != 0) {
              fprintf(stderr, "Error creating file in Spam Mailbox!\n");
              unlink(tmp_file);
              exit(EX_TEMPFAIL);
            }
          } else {
            if(send_mail_box(virus_file, tmp_file, mailfrom, iprelay, virus_ret)
               != 0) {
              fprintf(stderr, "Error creating file in Virus Mailbox!\n");
              unlink(tmp_file);
              exit(EX_TEMPFAIL);
            }
          }
        } else {
          if(allinone > 0) {
            if(maildir_put(final_msg, tmp_file, mailfrom, iprelay, virus_ret)
               != 0) {
              fprintf(stderr, "Error creating file in Virus Mailbox!\n");
              unlink(tmp_file);
              if(sendmail > 0)
                exit(EX_TEMPFAIL);
              else if(qmail_queue > 0)
                exit(53);
              else
                exit(0);
            }
          } else {
            if(maildir_put(virus_file, tmp_file, mailfrom, iprelay, virus_ret)
               != 0) {
              fprintf(stderr, "Error creating file in Virus Mailbox!\n");
              unlink(tmp_file);
              if(sendmail > 0)
                exit(EX_TEMPFAIL);
              else if(qmail_queue > 0)
                exit(53);
              else
                exit(0);
            }
          }
        }
      }
      unlink(tmp_file);

      if(DEBUG) {
        fprintf(stderr, "SpamMatch:  %s\n", log_info);
        fprintf(stderr, "SpamInfo:   %s\n", matches[match]);
        fprintf(stderr, "VirusType:  %s\n", virus_type);
        fprintf(stderr, "VirusState: %s\n", viruses[virus_ret]);
        fprintf(stderr, "\n");
      }

      if(use_log > 0) {
        if(match != 0 && log_info != NULL)
          logging(log_info);

        log_info = malloc(strlen(tmp_file) + strlen(virus_type) + 29 + 1);
        if(log_info != NULL) {
          snprintf(log_info, (strlen(tmp_file) + strlen(virus_type) + 29 + 1),
                   "VIRUS SCAN(%s) infected with (%s)", tmp_file, virus_type);

          logging(log_info);
        }
      }

      if(sendmail > 0 && bouncemsg > 0)
        exit(EX_NOUSER);
      else if(sendmail > 0)
        exit(0);
      else if(qmail_queue > 0 && virus_alert > 0)
        exit(31);
      else if(virus_alert > 0)
        exit(100);
      else
        exit(99);
    }
  }
  if(match > 0) {
    /* Message is Spam */
    if(bouncemsg > 0)
      fprintf(stdout, "%s\n", bounce_msg);

    if(store_email > 0) {
      if(use_maildir < 1 && sendmail > 0) {
        if(allinone > 0) {
          if(send_mail_box(mbox, tmp_file, mailfrom, iprelay, virus_ret)
             != 0) {
            fprintf(stderr, "Error creating file in Spam Mailbox!\n");
            unlink(tmp_file);
            exit(EX_TEMPFAIL);
          }
        } else {
          if(send_mail_box(spam_file, tmp_file, mailfrom, iprelay, virus_ret)
             != 0) {
            fprintf(stderr, "Error creating file in Spam Mailbox!\n");
            unlink(tmp_file);
            exit(EX_TEMPFAIL);
          }
        }
      } else {
        if(allinone > 0) {
          if(maildir_put(final_msg, tmp_file, mailfrom, iprelay, virus_ret)
             != 0) {
            fprintf(stderr, "Error creating file in Spam Mailbox!\n");
            unlink(tmp_file);
            if(sendmail > 0)
              exit(EX_TEMPFAIL);
            else if(qmail_queue > 0)
              exit(53);
            else
              exit(0);
          }
        } else {
          if(maildir_put(spam_file, tmp_file, mailfrom, iprelay, virus_ret)
             != 0) {
            fprintf(stderr, "Error creating file in Spam Mailbox!\n");
            unlink(tmp_file);
            if(sendmail > 0)
              exit(EX_TEMPFAIL);
            else if(qmail_queue > 0)
              exit(53);
            else
              exit(0);
          }
        }
      }
    }
    unlink(tmp_file);

    if(DEBUG) {
      fprintf(stderr, "SpamMatch:  %s\n", log_info);
      fprintf(stderr, "SpamInfo:   %s\n", matches[match]);
      if(virusscan > 0) {
        fprintf(stderr, "VirusType:  %s\n", virus_type);
        fprintf(stderr, "VirusState: %s\n", viruses[virus_ret]);
      }
      fprintf(stderr, "\n");
    }

    if(use_log > 0)
      logging(log_info);

    /* Block email */
    if(sendmail > 0 && bounce_msg > 0)
      exit(EX_NOUSER);
    else if(sendmail > 0)
      exit(0);
    else if(qmail_queue > 0 && bouncemsg > 0)
      exit(31);
    else if(bouncemsg > 0)
      exit(100);
    else
      exit(99);
  } else {
    /* Message is OK and CLEAN */
    if(use_maildir < 1 && sendmail > 0) {
      if(send_mail_box(mbox, tmp_file, mailfrom, iprelay, virus_ret) != 0) {
        unlink(tmp_file);
        exit(EX_TEMPFAIL);
      }
    } else if(sendmail > 0 || allinone > 0) {
      if(maildir_put(final_msg, tmp_file, mailfrom, iprelay, virus_ret)
         != 0) {
        fprintf(stderr, "Error writing maildir!\n");
        unlink(tmp_file);
        exit(EX_TEMPFAIL);
      }
    }

    unlink(tmp_file);
    /* We wrote it out, so exit silently */
    if(allinone > 0 && sendmail < 1)
      exit(99);

    /* Permit email through */
    exit(0);
  }
}

int send_mail_box(char *mbox, char *tmpfile, char *mailfrom, char *iprelay,
                  int virus_ret)
{
  extern char *version;
  int eoh = 0;
  FILE *tmp, *mb;
  char *buffer;
  int i;

  buffer = malloc(255 + 1);
  if(buffer == NULL)
    return 1;

  tmp = fopen(tmpfile, "r");
  if(tmp == NULL)
    return 1;

  /* Lock Mailbox */
  if(mbox_lock(mbox) != 0)
    return 1;

  mb = fopen(mbox, "a");
  if(mb == NULL)
    return 1;

  if(match == 0) {
    while(fgets(buffer, 255, tmp) != '\0') {
      fputs(buffer,mb);
    }
  } else {
    while(fgets(buffer, 255, tmp) != '\0') {
      /* Clean Attachment */
      if(eoh == 1 && disinfect > 0 && found_virus == 1 &&
         strncmp(buffer, "Content-Type:", 13) == 0 &&
         strncmp(buffer, "Content-Type: text", 18) != 0)
        break;

      if(allinone > 0 && eoh == 0 && found_virus > 0 &&
         (strncmp(buffer, "Subject:", 8) == 0 ||
          strncmp(buffer, "SUBJECT:", 8) == 0 ||
          strncmp(buffer, "subject:", 8) == 0)) {
        fprintf(mb, "Subject: VIRUS[%s %s] ", virus_type, viruses[virus_ret]);
        for(i = 8; buffer[i] != '\n' && buffer[i] != '\0'; i++)
          fputc(buffer[i],mb);
        fputc('\n',mb);
        continue;
      }
      if(allinone > 0 && eoh == 0 && match > 0 &&
         (strncmp(buffer, "Subject:", 8) == 0 ||
          strncmp(buffer, "SUBJECT:", 8) == 0 ||
          strncmp(buffer, "subject:", 8) == 0)) {
        fprintf(mb, "Subject: SPAM[%s] ", matches[match]);
        for(i = 8; buffer[i] != '\n' && buffer[i] != '\0'; i++)
          fputc(buffer[i],mb);
        fputc('\n',mb);
        continue;
      }

      /* Insert X-BlackHole: Headers */
      if(eoh == 0 && strncmp(buffer, "\n", 1) == 0) {
        fprintf(mb, "X-BlackHole: Version %s by Chris Kennedy (C) 2002\n",
                version);
        fprintf(mb, "X-BlackHole-Sender: %s\n", mailfrom);
        fprintf(mb, "X-BlackHole-Relay: %s\n", iprelay);
        fprintf(mb, "X-BlackHole-Match: %s\n", matches[match]);
        fprintf(mb, "X-BlackHole-Info: %s\n", log_info);
        if(virusscan > 0) {
          fprintf(mb, "X-BlackHole-Virus-Status: %s\n", viruses[virus_ret]);
          fprintf(mb, "X-BlackHole-Virus-Type: %s\n", virus_type);
        }
        eoh = 1;
      }
      fputs(buffer,mb);
    }
  }
  fputc('\n',mb);
  fclose(mb);

  /* Unlock Mailbox */
  if(mbox_unlock(mbox) != 0)
    return 1;

  fclose(tmp);
  return 0;
}

int mbox_lock(char *mbox)
{
  char *maillock;
  extern char **environ;
  int pid, status;

  maillock = malloc(strlen(mbox) + 5 + 1);
  if(maillock == NULL)
    return 1;
  snprintf(maillock, (strlen(mbox) + 5 + 1), "%s.lock", mbox);

  /* Fork for Lockfile */
  pid = fork();
  if(pid == -1)
    return 0;
  if(pid == 0) {
    char *lock_args[3] = {
      lockfile,
      maillock,
      '\0'
    };

    if(setgid_sendmail > 0) {
      lock_args[1] = malloc(3 + 1);
      strcpy(lock_args[1], "-ml");
      lock_args[2] = '\0';
    }

    /* Run Command */
    execve(lockfile, lock_args, environ);
    exit(127);
  }
  do {
    if(waitpid(pid, &status, 0) == -1) {
      if(errno != EINTR)
        return -1;
    } else {
      status >>= 8;
      status &= 0xFF;
      if(status != 0) {
        fprintf(stderr, "%s: Not able to run lockfile to lock! %s:%d\n",
                __FILE__, __FILE__, __LINE__);
        return 1;
      }
      break;
    }
  } while(1);


  free(maillock);
  return 0;
}

int mbox_unlock(char *mbox)
{
  char *maillock;
  extern char **environ;
  int pid, status;

  maillock = malloc(strlen(mbox) + 5 + 1);
  if(maillock == NULL)
    return 1;
  snprintf(maillock, (strlen(mbox) + 5 + 1), "%s.lock", mbox);

  /* If we don't have to don't run lockfile again */
  if(setgid_sendmail < 1) {
    if(unlink(maillock) != 0)
      return 1;
    else
      return 0;
  }

  /* Fork for Lockfile */
  pid = fork();
  if(pid == -1)
    return 0;
  if(pid == 0) {
    char *lock_args[] = {
      lockfile,
      "-mu",
      '\0'
    };

    /* Run Command */
    execve(lockfile, lock_args, environ);
    exit(127);
  }
  do {
    if(waitpid(pid, &status, 0) == -1) {
      if(errno != EINTR)
        return -1;
    } else {
      status >>= 8;
      status &= 0xFF;
      if(status != 0) {
        fprintf(stderr, "%s: Not able to run lockfile to unlock! %s:%d\n",
                __FILE__, __FILE__, __LINE__);
        /* Last attempt to remove */
        if(unlink(maillock) != 0)
          return 1;
      }
      break;
    }
  } while(1);

  free(maillock);
  return 0;
}

int maildir_put(char *vfile, char *tfile, char *mailfrom,
                char *iprelay, int virus_ret)
{
  extern int found_virus;
  extern char *version;
  int eoh = 0, i;

  FILE *vf, *tf;
  char *buffer;

  buffer = malloc(255 + 1);
  if(buffer == NULL)
    return 1;

  tf = fopen(tfile, "r");
  if(tf == NULL)
    return 1;

  vf = fopen(vfile, "w");
  if(vf == NULL)
    return 1;

  while(fgets(buffer, 255, tf) != '\0') {
    /* Clean Attachment */
    if(eoh == 1 && disinfect > 0 && found_virus == 1 &&
       strncmp(buffer, "Content-Type:", 13) == 0 &&
       strncmp(buffer, "Content-Type: text", 18) != 0)
      break;

    if(allinone > 0 && eoh == 0 && found_virus > 0 &&
       (strncmp(buffer, "Subject:", 8) == 0 ||
        strncmp(buffer, "SUBJECT:", 8) == 0 ||
        strncmp(buffer, "subject:", 8) == 0)) {
      fprintf(vf, "Subject: VIRUS[%s %s] ", virus_type, viruses[virus_ret]);
      for(i = 8; buffer[i] != '\n' && buffer[i] != '\0'; i++)
        fputc(buffer[i],vf);
      fputc('\n',vf);
      continue;
    }
    if(allinone > 0 && eoh == 0 && match > 0 &&
       (strncmp(buffer, "Subject:", 8) == 0 ||
        strncmp(buffer, "SUBJECT:", 8) == 0 ||
        strncmp(buffer, "subject:", 8) == 0)) {
      fprintf(vf, "Subject: SPAM[%s] ", matches[match]);
      for(i = 8; buffer[i] != '\n' && buffer[i] != '\0'; i++)
        fputc(buffer[i],vf);
      fputc('\n',vf);
      continue;
    }

    /* Insert X-BlackHole: Headers */
    if(eoh == 0 && strncmp(buffer, "\n", 1) == 0) {
      fprintf(vf, "X-BlackHole: Version %s by Chris Kennedy (C) 2002\n",
              version);
      fprintf(vf, "X-BlackHole-Sender: %s\n", mailfrom);
      fprintf(vf, "X-BlackHole-Relay: %s\n", iprelay);
      fprintf(vf, "X-BlackHole-Match: %s\n", matches[match]);
      fprintf(vf, "X-BlackHole-Info: %s\n", log_info);
      if(virusscan > 0) {
        fprintf(vf, "X-BlackHole-Virus-Status: %s\n", viruses[virus_ret]);
        fprintf(vf, "X-BlackHole-Virus-Type: %s\n", virus_type);
      }
      eoh = 1;
    }
    fputs(buffer,vf);
  }
  fputc('\n',vf);
  fclose(tf);
  fclose(vf);

  /* OK */
  return 0;
}
