/* checkbody.c */
/*
   Copyright (C) 2002
        Chris Kennedy, The Groovy Organization.

   The Blackhole is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Blackhole is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   For a copy of the GNU Library General Public License
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  or go to http://www.gnu.org
*/
#include <stdio.h>
#include <string.h>
#include <pcre.h>
#include <stdlib.h>
#include "body_patterns.h"

int check_line(char *, char *);

extern int DEBUG;
extern int SPAM_BODY;
extern int PORN_BODY;
extern int RACIST_BODY;
extern int MY_BODY;
extern float my_score, spam_score, porn_score, racist_score;
extern float my_thresh, spam_thresh, porn_thresh, racist_thresh;
extern char *log_info;

extern char *mybody[];

#define OVECCOUNT 30            /* multiple of 3 */

int check_body(char *line)
{
  int i;

  /* Body checking categories */
  if(MY_BODY > 0) {
    if(MY_BODY > 1)
      my_thresh = MY_BODY;
    for(i = 0; mybody[i] != NULL; i++) {
      if(check_line(mybody[i], line)) {
        /* Match My Custom pattern */
        my_score += 1.0;
        if(my_score >= my_thresh) {
          if(DEBUG)
            fprintf(stderr,
                    " Match MY body pattern:\n  %s\n  Line: %s\n",
                    mybody[i], line);

          log_info = malloc(sizeof(int) + sizeof(float) +
                            strlen(mybody[i]) + 33 + 1);
          if(log_info == NULL)
            return 0;

          snprintf(log_info,
                   (sizeof(int) + sizeof(float) + strlen(mybody[i]) +
                    strlen(line) + 33 + 1),
                   "BODY PATTERN MY(+%d) %.2f Match(%s)",
                   i, my_score, mybody[i]);

          return MY;
        } else
          return 0;
      }
    }
  }
  if(SPAM_BODY > 0) {
    if(SPAM_BODY > 1)
      spam_thresh = SPAM_BODY;
    for(i = 0; spam_pattern[i].pattern != NULL; i++) {
      if(check_line(spam_pattern[i].pattern, line)) {
        /* Match Spam */
        spam_score += spam_pattern[i].score;
        if(spam_score >= spam_thresh) {
          if(DEBUG)
            fprintf(stderr,
                    " Match SPAM body pattern:\n  %s\n  Line: %s\n",
                    spam_pattern[i].pattern, line);

          log_info = malloc(sizeof(int) + sizeof(float) +
                            strlen(spam_pattern[i].pattern) + 33 + 1);
          if(log_info == NULL)
            return 0;

          snprintf(log_info,
                   (sizeof(int) + sizeof(float) +
                    strlen(spam_pattern[i].pattern) + strlen(line) + 33 + 1),
                   "BODY PATTERN SPAM(+%d) %.2f Match(%s)", i, spam_score,
                   spam_pattern[i].pattern);

          return SPAM;
        } else
          return 0;
      }
    }
  }
  if(PORN_BODY > 0) {
    if(PORN_BODY > 1)
      porn_thresh = PORN_BODY;
    for(i = 0; porn_pattern[i].pattern != NULL; i++) {
      if(check_line(porn_pattern[i].pattern, line)) {
        /* Match Porn */
        porn_score += porn_pattern[i].score;
        if(porn_score >= porn_thresh) {
          if(DEBUG)
            fprintf(stderr,
                    " Match PORN body pattern:\n  %s\n  Line: %s\n",
                    porn_pattern[i].pattern, line);

          log_info = malloc(sizeof(int) + sizeof(float) +
                            strlen(porn_pattern[i].pattern) + 33 + 1);
          if(log_info == NULL)
            return 0;

          snprintf(log_info,
                   (sizeof(int) + sizeof(float) +
                    strlen(porn_pattern[i].pattern) + strlen(line) + 33 + 1),
                   "BODY PATTERN PORN(+%d) %.2f Match(%s)", i, porn_score,
                   porn_pattern[i].pattern);

          return PORN;
        } else
          return 0;
      }
    }
  }
  if(RACIST_BODY > 0) {
    if(RACIST_BODY > 1)
      racist_thresh = RACIST_BODY;
    for(i = 0; racist_pattern[i].pattern != NULL; i++) {
      if(check_line(racist_pattern[i].pattern, line)) {
        /* Match Racist */
        racist_score += racist_pattern[i].score;
        if(racist_score >= racist_thresh) {
          if(DEBUG)
            fprintf(stderr,
                    " Match RACIST body pattern:\n  %s\n  Line: %s\n",
                    racist_pattern[i].pattern, line);

          log_info = malloc(sizeof(int) + sizeof(float) +
                            strlen(racist_pattern[i].pattern) + strlen(line) +
                            33 + 1);
          if(log_info == NULL)
            return 0;

          snprintf(log_info,
                   (sizeof(int) + sizeof(float) +
                    strlen(racist_pattern[i].pattern) + 35 + 1),
                   "BODY PATTERN RACIST(+%d) %.2f Match(%s)", i, racist_score,
                   racist_pattern[i].pattern);

          return RACIST;
        } else
          return 0;
      }
    }
  }

  /* No Match after all patterns checked against threshholds */
  return 0;
}

int check_line(char *pattern, char *line)
{
  pcre *re;
  const char *error;
  int erroffset;
  int ovector[OVECCOUNT];
  int rc;

  re = pcre_compile(pattern, 0, &error, &erroffset, NULL);
  if(re == NULL)
    return 0;
  rc = pcre_exec(re, NULL, line, (int) strlen(line), 0, 0, ovector, OVECCOUNT);
  pcre_free(re);

  /* Check return value */
  if(rc < 0) {
    switch (rc) {
    case PCRE_ERROR_NOMATCH:
      break;
    default:
      if(DEBUG)
        fprintf(stderr, "Matching Error %d\n", rc);
      break;
    }
    /* No Match */
    return 0;
  }

  /* Match */
  return 1;
}
