/* checkreverse.c */
/*
   Copyright (C) 2002
        Chris Kennedy, The Groovy Organization.

   The Blackhole is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Blackhole is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   For a copy of the GNU Library General Public License
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  or go to http://www.gnu.org
*/

/* some DNS code borrowed from: 
 * rblcheck 1.4 - Command-line interface to Paul Vixie's RBL filter.
 * Copyright (C) 1997, Edward S. Marshall <emarshal@logic.net>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/socket.h>


#ifndef PACKETSZ
#define PACKETSZ 512
#endif

extern int DEBUG;
extern char *log_info;

int check_reverse(char *iprelay, char *hostrelay)
{
  u_char fixedans[PACKETSZ];
  u_char *answer;
  int length;
  struct hostent *host_ent;
  struct in_addr address;

  /* DNS query. */
  res_init();
  answer = fixedans;
  length = res_query(hostrelay, C_IN, T_A, answer, PACKETSZ);
  if(length == -1)
    length = res_query(hostrelay, C_IN, T_MX, answer, PACKETSZ);

  /* Not a valid hostname */
  if(length == -1) {
    /* Bad Hostname */
    if(DEBUG)
      fprintf(stderr, "  BAD REVERSE DNS(domain/host) Match %s %s\n",
              iprelay, hostrelay);

    log_info = malloc(strlen(iprelay) + 48 + 1);
    if(log_info == NULL)
      return (0);

    snprintf(log_info,
             (strlen(iprelay) + 48 + 1),
             "BAD REVERSE DNS Match(no such domain/host) in (%s)", iprelay);

    return 0;
  }

  if(length > PACKETSZ) {
    answer = malloc(length);
    length = res_query(hostrelay, C_IN, T_A, answer, length);
    if(length == -1)
      length = res_query(hostrelay, C_IN, T_MX, answer, PACKETSZ);
    if(length == -1) {
      /* Bad Hostname */
      if(DEBUG)
        fprintf(stderr,
                "  BAD REVERSE DNS(domain/host) Big Match %s %s\n",
                iprelay, hostrelay);

      log_info = malloc(strlen(iprelay) + 48 + 1);
      if(log_info == NULL)
        return (0);

      snprintf(log_info,
               (strlen(iprelay) + 48 + 1),
               "BAD REVERSE DNS Match(no such domain/host) in (%s)", iprelay);

      return 0;
    }
  }

  /* Get Hostname from IP */
  host_ent = gethostbyname(hostrelay);
  if(host_ent == NULL)
    return 1;
  memcpy(&address, host_ent->h_addr, 4);

  if(DEBUG) {
    fprintf(stderr, "Hostname: %s Resolves to %s compared with %s.\n",
            hostrelay, inet_ntoa(address), iprelay);
  }

  /* Compare relay ip with ip of hostname it resolves to in reverse */
  if(strncmp(iprelay, inet_ntoa(address), strlen(iprelay)) != 0) {
    /* Bad Hostname */
    if(DEBUG)
      fprintf(stderr,
              "  BAD REVERSE DNS(domain/host) Match %s %s\n",
              iprelay, hostrelay);

    log_info = malloc(strlen(iprelay) + 48 + 1);
    if(log_info == NULL)
      return (0);

    snprintf(log_info,
             (strlen(iprelay) + 48 + 1),
             "BAD REVERSE DNS Match(no such domain/host) in (%s)", iprelay);

    return 0;
  }

  /* Valid Domain */
  return 1;
}
