/* logging.c */
/*
   Copyright (C) 2002
        Chris Kennedy, The Groovy Organization.

   The Blackhole is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Blackhole is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   For a copy of the GNU Library General Public License
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  or go to http://www.gnu.org
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/syslog.h>
#include <mysql/mysql.h>

int bh_syslog(char *);
int bh_mysqllog(char *);
int bh_stderr(char *);
int bh_stdout(char *);

extern char *username;
extern char *timestamp;
extern char *hostname;
extern char *mysql_user;
extern char *mysql_pass;
extern char *mysql_host;
extern char *mysql_domain;

extern int log_type;
enum log_types
{
  error = 0,
  output,
  syslg,
  mysql
};

int logging(char *line)
{
  enum log_types ltype = log_type;
  if(ltype == mysql)
    bh_mysqllog(line);
  else if(ltype == syslg)
    bh_syslog(line);
  else if(ltype == output)
    bh_stdout(line);
  else
    bh_stderr(line);

  return 0;
}

int bh_syslog(char *line)
{
  openlog("BlackHole", 0, LOG_MAIL);
  syslog(LOG_INFO, "%s", line);
  closelog();

  return 0;
}

/* table format:  timestamp, username, domain, entry */
int bh_mysqllog(char *line)
{
  MYSQL *connection;
  MYSQL mysql;
  MYSQL_RES *results;
  char *query_start = "INSERT into log set username='";
  char *query_two = "',domain='";
  char *query_three = "',entry='";
  char *query;

  if(!mysql_init(&mysql)) {
    fprintf(stderr, mysql_error(&mysql));
    return 1;
  }

  connection = mysql_real_connect(&mysql,
                                  mysql_host,
                                  mysql_user, mysql_pass,
                                  "BlackHole", 0, "", 0);

  if(connection == NULL) {
    fprintf(stderr, mysql_error(&mysql));
    fprintf(stderr, "\n");
    mysql_close(&mysql);
    return 1;
  }

  query = malloc(strlen(query_start) + strlen(username) + strlen(query_two) +
                 strlen(mysql_domain) + strlen(query_three) + strlen(line) + 1 +
                 1);

  snprintf(query, (strlen(query_start) + strlen(username) + strlen(query_two) +
                   strlen(mysql_domain) + strlen(query_three) + strlen(line) +
                   1 + 1), "%s%s%s%s%s%s'", query_start, username, query_two,
           mysql_domain, query_three, line);

  mysql_real_query(&mysql, query, strlen(query));
  results = mysql_use_result(&mysql);
  if(results == NULL) {
    return 1;
  }

  mysql_free_result(results);
  mysql_close(&mysql);

  return 0;
}

int bh_stdout(char *line)
{
  fprintf(stdout, "%s %s %s %s\n", timestamp, hostname, username, line);

  return 0;
}

int bh_stderr(char *line)
{
  fprintf(stderr, "%s %s %s %s\n", timestamp, hostname, username, line);

  return 0;
}
