/* mysqlconfig.c */
/*
   Copyright (C) 2002
        Chris Kennedy, The Groovy Organization.

   The Blackhole is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Blackhole is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   For a copy of the GNU Library General Public License
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  or go to http://www.gnu.org
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <mysql/mysql.h>
#include <time.h>

extern int DEBUG;

int get_config_tstamp(char *);

extern char *username;
extern char *hostname;
extern char *mysql_user;
extern char *mysql_pass;
extern char *mysql_host;
extern char *mysql_domain;
extern char *config_file;

int mysql_config(void)
{
  int i;
  int num_fields;
  char *fields[255];
  MYSQL *connection;
  MYSQL mysql;
  MYSQL_RES *results;
  MYSQL_FIELD *field;
  MYSQL_ROW row;
  char *query_start = "SELECT * from BlackHole where username=";
  char *query_end = "and domain=";
  char *query_tstamp = "SELECT timestamp from BlackHole where username=";
  char *query;
  char *tquery;
  int loc_cfg_mtime;
  int rem_cfg_mtime;
  FILE *config_tmp = NULL;
  FILE *lck = NULL;
  char *mysql_lock = ".bh_mysql";
  char *mysql_lock_link;
  pid_t pid;
  time_t now;

  /* get current UNIX time and PID */
  time(&now);
  pid = getpid();

  mysql_lock_link = malloc(strlen(mysql_lock) + strlen(hostname) + 35 + 1);
  if(mysql_lock_link == NULL) {
    fprintf(stderr, "%s: Out of memory! %s:%d\n", __FILE__, __FILE__, __LINE__);
    return 1;
  }
  snprintf(mysql_lock_link, 
           (strlen(mysql_lock) + strlen(hostname) + 35 + 1),
           "%s.%d.%d.%s", mysql_lock, (int) now, pid, hostname);

  lck = fopen(mysql_lock_link, "w");
  if(lck == NULL) {
    fprintf(stderr, "%s: Out of memory! %s:%d\n", __FILE__, __FILE__, __LINE__);
    return 1;
  }

  /* try linking lock file */
  if(link(mysql_lock_link, mysql_lock) != 0) {
    fclose(lck);
    unlink(mysql_lock_link);
    return 1;
  } else 
    fclose(lck);

  if(!mysql_init(&mysql)) {
    fprintf(stderr, mysql_error(&mysql));
    unlink(mysql_lock);
    unlink(mysql_lock_link);
    return 1;
  }

  connection = mysql_real_connect(&mysql,
                                  mysql_host,
                                  mysql_user, mysql_pass,
                                  "BlackHole", 0, "", 0);

  if(connection == NULL) {
    fprintf(stderr, mysql_error(&mysql));
    fprintf(stderr, "\n");
    mysql_close(&mysql);
    unlink(mysql_lock);
    unlink(mysql_lock_link);
    return 1;
  }

  tquery =
    malloc(strlen(query_tstamp) + strlen(username) +
           strlen(query_end) + strlen(mysql_domain) + 6);
  snprintf(tquery,
           (strlen(query_tstamp) + strlen(username) +
            strlen(query_end) + strlen(mysql_domain) + 6),
           "%s'%s' %s'%s'", query_tstamp, username, query_end, mysql_domain);

  /* Make MySQL timestamp Query */
  mysql_real_query(&mysql, tquery, strlen(tquery));
  results = mysql_use_result(&mysql);
  if(results == NULL) {
    mysql_close(&mysql);
    unlink(mysql_lock);
    unlink(mysql_lock_link);
    return 1;
  }
  row = mysql_fetch_row(results);
  if(row == NULL)
    goto exit_bad;

  /* Get the array of lengths for each field */
  num_fields = mysql_num_fields(results);
  i = 0;
  while((field = mysql_fetch_field(results)) != NULL) {
    /* Skip username, we know it already */
    if(strcmp(field->name, "timestamp") == 0) {
      rem_cfg_mtime = (int) atoi(row[i]);
      loc_cfg_mtime = get_config_tstamp(config_file);
      if(DEBUG)
        fprintf(stderr, "Local Config last modified at: %d\n", loc_cfg_mtime);
      if(DEBUG)
        fprintf(stderr, "MySQL Config last modified at: %d\n", rem_cfg_mtime);

      /* New mysql entry, no timestamp yet */
      if(rem_cfg_mtime == 0) {
        goto exit_bad;
        /* local config is newer than mysql, cached */
      } else if(rem_cfg_mtime <= loc_cfg_mtime) {
        goto exit_bad;
        /* remote mysql config is newer than local */
      } else if(rem_cfg_mtime > loc_cfg_mtime) {
        /* Open tmp lock file */
        config_tmp = fopen(mysql_lock_link, "w");
        if(config_tmp == NULL)
          goto exit_bad;

        if(DEBUG)
          fprintf(stderr, "Local cache is stale, retrieving new config...\n");
        break;
      }
      goto exit_bad;
    }
    i++;
  }
  mysql_free_result(results);

  query =
    malloc(strlen(query_start) + strlen(username) +
           strlen(query_end) + strlen(mysql_domain) + 6);
  snprintf(query,
           (strlen(query_start) + strlen(username) +
            strlen(query_end) + strlen(mysql_domain) + 6),
           "%s'%s' %s'%s'", query_start, username, query_end, mysql_domain);

  /* Make MySQL Query */
  mysql_real_query(&mysql, query, strlen(query));
  results = mysql_use_result(&mysql);
  if(results == NULL)
    goto exit_bad;

  row = mysql_fetch_row(results);
  if(row == NULL)
    goto exit_bad;

  /* Get the array of lengths for each field */
  num_fields = mysql_num_fields(results);

  /* Walk through results */
  i = 0;
  while((field = mysql_fetch_field(results)) != NULL) {
    /* Skip username, we know it already */
    if(strcmp(field->name, "username") == 0) {
      i++;
      continue;
      /* Get timestamp */
    } else if(strcmp(field->name, "timestamp") == 0) {
      i++;
      continue;
    } else if(strcmp(field->name, "domain") == 0) {
      i++;
      continue;
    }

    /* The other fields */
    fields[i] = malloc(strlen(field->name) + 1);
    if(fields[i] == NULL) {
      goto exit_bad;
    }
    strcpy(fields[i], field->name);

    /* Print out row field and value */
    if(row[i] != NULL) {
      fprintf(config_tmp, "[%s]\n", fields[i]);
      fprintf(config_tmp, "%s\n", row[i]);
    } else {
      fprintf(config_tmp, "[%s]\n", fields[i]);
    }

    i++;
  }

  /* Close tmp config file */
  fclose(config_tmp);

  /* Rename tmp file to config */
  if(rename(mysql_lock_link, config_file) == -1) {
    fprintf(stderr, "Could not rename tmp file, exiting...\n");
    goto exit_bad;
  }
  goto exit_good;

exit_bad:
  mysql_free_result(results);
  mysql_close(&mysql);
  unlink(mysql_lock);
  unlink(mysql_lock_link);
  return 1;

exit_good:
  mysql_free_result(results);
  mysql_close(&mysql);
  unlink(mysql_lock);
  unlink(mysql_lock_link);
  return 0;
}

int get_config_tstamp(char *filename)
{
  struct stat buf;

  if(stat(filename, &buf) == 0) {
    if(buf.st_mtime != 0)
      return buf.st_mtime;
  }
  return -1;
}
