/* rbllookup.c */
/*
   Copyright (C) 2002
        Chris Kennedy, The Groovy Organization.

   The Blackhole is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Blackhole is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   For a copy of the GNU Library General Public License
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  or go to http://www.gnu.org
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int rblcheck(int, int, int, int, char *);

extern int DEBUG;
extern char *log_info;

int rbllookup(char iprelay[16], char *rblhosts[],
              int maxlevel, int STOP_WHEN_FOUND)
{
  int a, b, c, d;
  int rblfiltered = 0;
  int response = 0;
  int i;

  /* Check for Valid IP Address */
  if(sscanf(iprelay, "%d.%d.%d.%d", &a, &b, &c, &d) != 4 ||
     a < 0 || a > 255 || b < 0 || b > 255 || c < 0 || c > 255 ||
     d < 0 || d > 255) {
    return 0;
  }

  /* Go through RBL Hosts,  */
  for(i = 0; rblhosts[i] != NULL; i++) {
    if(DEBUG)
      fprintf(stderr, " (%d.) %-25s", i, rblhosts[i]);

    response = rblcheck(a, b, c, d, rblhosts[i]);
    if(response == 1) {
      rblfiltered++;

      if(DEBUG)
        fprintf(stderr, " Match (+%d) %d.%d.%d.%d (%s).\n",
                rblfiltered, a, b, c, d, rblhosts[i]);

      if(STOP_WHEN_FOUND > 0) {
        log_info = malloc(strlen(rblhosts[i]) + (sizeof(int) * 2) + 24 + 1);
        if(log_info == NULL)
          return 1;

        snprintf(log_info,
                 (strlen(rblhosts[i]) + (sizeof(int) * 2) + 24 + 1),
                 "Match(+%d) Level(%d) Host (%s)", rblfiltered, i, rblhosts[i]);
        return 1;
      }
    } else if(DEBUG)
      fprintf(stderr, "\n");

    if(i >= maxlevel && STOP_WHEN_FOUND > 0)
      break;

    if(STOP_WHEN_FOUND < 1) {
      if(rblfiltered >= maxlevel) {
        log_info = malloc(strlen(rblhosts[i]) + (sizeof(int) * 2) + 24 + 1);
        if(log_info == NULL)
          return 1;

        snprintf(log_info,
                 (strlen(rblhosts[i]) + (sizeof(int) * 2) + 24 + 1),
                 "Match(+%d) Level(%d) Host (%s)", rblfiltered, i, rblhosts[i]);
        return 1;
      }
    }
  }
  return 0;
}

