/* Sophos Virus Scanning Code */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "csavi2c.h"
#include "sophos.h"

extern int DEBUG;
extern char *progname;
extern char *virus_type;
extern int found_virus;

void sophos_infected(HRESULT, CIEnumSweepResults *);

int sophos(char *filename)
{
  CISavi2 *pSAVI;
  CISweepClassFactory2 *pFactory;
  CIEnumSweepResults *scan_results = NULL;
  HRESULT hr;
  int i;

  /* Initialize the Sophos Scan Engine */
  hr =
    DllGetClassObject((REFIID) & SOPHOS_CLSID_SAVI2,
                      (REFIID) & SOPHOS_IID_CLASSFACTORY2, (void **) &pFactory);

  if(!(hr == SOPHOS_S_OK)) {
    if(DEBUG)
      fprintf(stderr, "ERROR: Could not initialize SAVI class/object\n");
    return 1;
  } else {
    hr =
      pFactory->pVtbl->CreateInstance(pFactory, NULL, &SOPHOS_IID_SAVI2,
                                      (void **) &pSAVI);

    pFactory->pVtbl->Release(pFactory);

    /* If Sophos setup is ok, scan file */
    if(hr == SOPHOS_S_OK) {
      hr = pSAVI->pVtbl->InitialiseWithMoniker(pSAVI, progname);
      if(SOPHOS_FAILED(hr)) {
        fprintf(stderr, "ERROR: Failed to initialize SAVI [%ld]", (long) hr);
        pSAVI->pVtbl->Release(pSAVI);
        pSAVI = NULL;
        return 1;
      }

      /* set Recursion depth */
      hr = pSAVI->pVtbl->SetConfigValue(pSAVI,
                                        SOPHOS_MAX_RECURSION_DEPTH,
                                        SOPHOS_TYPE_U16, "16");
      if(SOPHOS_FAILED(hr))
        fprintf(stderr, "ERROR: Failed to set %s to \"16\"\n",
                SOPHOS_MAX_RECURSION_DEPTH);

      /* Configure Sophos */
      for(i = 0; sophos_settings[i].option != NULL; i++) {
        if(DEBUG)
          fprintf(stderr, "setting %s to \"%s\"\n",
                  sophos_settings[i].option, sophos_settings[i].value);
        hr = pSAVI->pVtbl->SetConfigValue(pSAVI,
                                          sophos_settings[i].option,
                                          SOPHOS_TYPE_U32,
                                          sophos_settings[i].value);
        if(SOPHOS_FAILED(hr))
          fprintf(stderr, "ERROR: Failed to set %s to \"%s\"\n",
                  sophos_settings[i].option, sophos_settings[i].value);
      }

      /* Scan File */
      hr =
        pSAVI->pVtbl->SweepFile(pSAVI, filename,
                                (REFIID) & SOPHOS_IID_ENUM_SWEEPRESULTS,
                                (void **) &scan_results);

      /* Check Results */
      switch (hr) {
      case SOPHOS_S_OK:
        break;

      case SOPHOS_SAVI2_ERROR_VIRUSPRESENT:
        sophos_infected(hr, scan_results);
        break;

      default:
        break;
      }


      if(pSAVI) {
        pSAVI->pVtbl->Terminate(pSAVI);
        pSAVI->pVtbl->Release(pSAVI);
      }
    }
  }
  return 0;
}

void sophos_infected(HRESULT hr, CIEnumSweepResults * scan_results)
{
  OLECHAR virusName[256];
  CISweepResults *details = NULL;
  SOPHOS_ULONG fetched;

  /* Read the results */
  while(scan_results->pVtbl->
        Next(scan_results, 1, (void **) &details, &fetched) == SOPHOS_S_OK) {
    hr =
      details->pVtbl->GetVirusName(details, sizeof(virusName) - 1,
                                   (LPOLESTR) virusName, NULL);

    /* Failed at reading results */
    if(SOPHOS_FAILED(hr)) {
      if(DEBUG)
        fprintf(stderr, "Sophos ERROR: Could not get a virus name");
    }

    /* Setup external variables */
    found_virus = 1;
    virus_type = malloc(sizeof(virusName));
    if(virus_type != NULL)
      snprintf(virus_type, (sizeof(virusName) - 1), "%s", virusName);
  }
}
