/* virusscan.c */
/*
   Copyright (C) 2002
        Chris Kennedy, The Groovy Organization.

   The Blackhole is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Blackhole is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   For a copy of the GNU Library General Public License
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  or go to http://www.gnu.org
*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/types.h>

int get_file_size(char *);

extern int virus_checker;
extern int McAfee;
extern int SophosSDK;
#if VIRUS_SCANNER == 2
int sophos(char *);
#endif
extern int DEBUG;
extern int disinfect;
extern char *tmp_file;
extern char *spool_dir;
extern char *uvscan;
extern char *dat_dir;
extern char *virus_type;

extern int found_virus;

int virus_scan(void)
{
  int pid, status;
  FILE *f;
  char *tmp_log;
  char *line;
  pid_t mypid;
  time_t mytime;
  int virus_status = 0;

  if(DEBUG)
    fprintf(stderr, "Virus Scanning: %s\n", tmp_file);

  /* Skip over zero byte file */
  if(get_file_size(tmp_file) == 0) {
    if(DEBUG)
      fprintf(stderr, "Skipping Zero sized file: %s\n", tmp_file);
    return 0;
  }

  if(virus_checker == SophosSDK) {
    #if VIRUS_SCANNER == 2
    sophos(tmp_file); 
    #endif 
    if(found_virus == 1) {
      if(disinfect > 0) {
        status = 19;
        virus_status = 2;
      } else {
        status = 13;
        virus_status = 1;
      }
      if(DEBUG)
        fprintf(stderr, "  Match Virus Type: %s\n", virus_type);
    } else
      status = 0;
  } else if(virus_checker == McAfee) {
    /* Time and Pid for Filename */
    time(&mytime);
    mypid = getpid();

    /* line buffer for parsing Virus Output */
    line = malloc((255 * sizeof(char)) + 1);
    if(line == NULL) {
      fprintf(stderr,
              "%s: Out of memory! at %s:%d\n", __FILE__, __FILE__, __LINE__);
      return 0;
    }

    /* virus_type size */
    virus_type = malloc((60 * sizeof(char)) + 1);
    if(virus_type == NULL) {
      fprintf(stderr,
              "%s: Out of memory! at %s:%d\n", __FILE__, __FILE__, __LINE__);
      return 0;
    }

    /* Virus Scanner Output file */
    tmp_log = (char *) malloc((32 * sizeof(char)) + strlen(spool_dir) + 17);
    if(tmp_log == NULL) {
      fprintf(stderr,
              "%s: Out of memory! at %s:%d\n", __FILE__, __FILE__, __LINE__);
      return 0;
    }
    snprintf(tmp_log, ((32 * sizeof(char)) + strlen(spool_dir) + 17),
             "%s/scanner/%i.%i.virus", spool_dir, (int) mytime, mypid);

    /* Fork for Virus Scanner */
    pid = fork();
    if(pid == -1) {
      fprintf(stderr,
              "%s: Error Spawning Pid! %s:%d\n", __FILE__, __FILE__, __LINE__);
      return 0;
    }
    if(pid == 0) {
      extern char **environ;
      char *mcafee_args[] = {
        uvscan,
        "--unzip",
        "--exit-on-error",
        "--noboot",
        "--norename",
        "--noexpire",
        "--ignore-links",
        "--mime",
        "--dat",
        dat_dir,
        tmp_file,
        '\0'
      };

      /* Redirect scanner output to a tmp file */
      f = freopen(tmp_log, "w+", stdout);
      if(f == NULL) {
        fprintf(stderr, "%s: Not able to open tmp file! %s:%d\n",
                __FILE__, __FILE__, __LINE__);
        return 0;
      }

      /* Virus Scan the mime parts */
      execve(uvscan, mcafee_args, environ);

      /* Error if still here */
      fprintf(stderr, "Error Scanning\n");
      exit(127);
    }
    do {
      if(waitpid(pid, &status, 0) == -1) {
        if(errno != EINTR) {
          fprintf(stderr, "%s: Error Executing Scanner! %s:%d\n",
                  __FILE__, __FILE__, __LINE__);
          return -1;
        }
      } else {
        status >>= 8;
        status &= 0xFF;

        f = fopen(tmp_log, "r");
        if(f == NULL) {
          fprintf(stderr, "%s: Not able to open tmp file! %s:%d\n",
                  __FILE__, __FILE__, __LINE__);
          return 0;
        }
        /* Write Virus Scan results to file, then extract to variable */
        if(virus_checker == McAfee) {
          int i, j;
          if(status == 12 || status == 13 || status == 19) {
            found_virus = 1;
            if(disinfect > 0)
              virus_status = 2;
            else
              virus_status = 1;
            rewind(f);
            while(fgets(line, 255, f)) {
              if(!strncmp(line, "        Found", 13)) {
                strcpy(virus_type, line);
                break;
              }
            }
            for(i = 18, j = 0;
                virus_type[i] != '\0' && virus_type[i] != '\n'; i++) {
              if(virus_type[i] != ' ') {
                virus_type[j++] = virus_type[i];
              } else
                break;
            }
            virus_type[j] = '\0';
          }
        }
        fclose(f);
        unlink(tmp_log);

        if(DEBUG) {
          if(status == 0)
            fprintf(stderr, "File is Clean\n");
          else if(status == 12 || status == 13)
            fprintf(stderr, "Failed at Cleaning\n");
          else if(status == 19)
            fprintf(stderr, "Success at Cleaning\n");
          else
            fprintf(stderr, "Error with Scanner (%d)\n", status);
        }
        break;
      }
    } while(1);
  }

  /* Return virus code if found, else Clean */
  return virus_status;
}

int get_file_size(char *filename)
{
  struct stat buf;

  if(stat(filename, &buf) == 0) {
    if(buf.st_size == 0)
      return buf.st_size;
  }
  return -1;
}
